@@ -1,6 +1,5 @@
 /* handshake.c
  *
- * $Id: handshake.c,v 1.13 2002/05/29 22:57:36 _cvs_pont Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -22,32 +21,36 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <string.h>
+
 #include "handshake.h"
 
+#include "command.h"
 #include "compress.h"
 #include "connection.h"
-#include "debug.h"
 #include "format.h"
 #include "io.h"
 #include "keyexchange.h"
 #include "read_line.h"
 #include "read_packet.h"
 #include "ssh.h"
-#include "unpad.h"
 #include "version.h"
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-#include <string.h>
-
 #define GABA_DEFINE
 #include "handshake.h.x"
 #undef GABA_DEFINE
 
 #include "handshake.c.x"
 
-#if DATAFELLOWS_WORKAROUNDS
+/* At the moment, we don't have any bug-compatibility hacks. */
+#if 0
 /* Bug compatibility information. */
 struct compat_info
 {
@@ -58,18 +61,11 @@ struct compat_info
 static const struct compat_info
 compat[] =
   {
-    /* FIXME: Is there any 2.0.x without these bugs? */
-    { "2.0.", (PEER_SSH_DSS_KLUDGE | PEER_SERVICE_ACCEPT_KLUDGE
-	       | PEER_USERAUTH_REQUEST_KLUDGE | PEER_SEND_NO_DEBUG 
-	       | PEER_X11_OPEN_KLUDGE) },
-    { "2.1.0", (PEER_SSH_DSS_KLUDGE
-		| PEER_USERAUTH_REQUEST_KLUDGE | PEER_SEND_NO_DEBUG) },
-    { "3.0", PEER_SEND_NO_DEBUG },
     { NULL, 0 }
   };
     
 static enum peer_flag
-compat_peer_flags(UINT32 length, UINT8 *software)
+compat_peer_flags(uint32_t length, const uint8_t *software)
 {
   unsigned i;
   for (i = 0; compat[i].prefix; i++)
@@ -89,31 +85,28 @@ compat_peer_flags(UINT32 length, UINT8 *software)
   /* Default flags are 0 */
   return 0;
 }
-#else /* !DATAFELLOWS_WORKAROUNDS */
+#else
 #define compat_peer_flags(a,b) 0
-#endif /* !DATAFELLOWS_WORKAROUNDS */
+#endif
 
 /* GABA:
    (class
      (name connection_line_handler)
      (super line_handler)
      (vars
-       (connection object ssh_connection)
-       ; Needed for fallback.
-       (fd . int)
-       (fallback object ssh1_fallback)))
+       (connection object ssh_connection)))
 */
 
-/* Returns -1 if the line is not the start of a SSH handshake,
- * 0 if the line appears to be an SSH handshake, but with bogus version fields,
- * or 1 if the line was parsed sucessfully. */
+/* Returns -1 if the line is not the start of a SSH handshake, 0 if
+ * the line appears to be an SSH handshake, but with bogus version
+ * fields, or 1 if the line was parsed sucessfully. */
 static int
-split_version_string(UINT32 length, UINT8 *line,
-		     UINT32 *protover_len, UINT8 **protover,
-		     UINT32 *swver_len, UINT8 **swver,
-		     UINT32 *comment_len, UINT8 **comment)
+split_version_string(uint32_t length, const uint8_t *line,
+		     uint32_t *protover_len, const uint8_t **protover,
+		     uint32_t *swver_len, const uint8_t **swver,
+		     uint32_t *comment_len, const uint8_t **comment)
 {
-  UINT8 *sep;
+  uint8_t *sep;
 
   if (length < 4 || memcmp(line, "SSH-", 4) != 0)
     {
@@ -154,13 +147,14 @@ split_version_string(UINT32 length, UINT8 *line,
 static void
 do_line(struct line_handler **h,
 	struct read_handler **r,
-	UINT32 length,
-	UINT8 *line,
-	struct exception_handler *e)
+	uint32_t length,
+	const uint8_t *line,
+	/* FIXME: Do we need this argument at all? */
+	struct exception_handler *e UNUSED)
 {
   CAST(connection_line_handler, closure, *h);
-  UINT32 protover_len, swver_len, comment_len;
-  UINT8 *protover, *swver, *comment;
+  uint32_t protover_len, swver_len, comment_len;
+  const uint8_t *protover, *swver, *comment;
 
   struct ssh_connection *connection = closure->connection;
   int mode = connection->flags & CONNECTION_MODE;
@@ -181,33 +175,16 @@ do_line(struct line_handler **h,
 	if ( ((protover_len >= 3) && !memcmp(protover, "2.0", 3))
 	     || ((protover_len == 4) && !memcmp(protover, "1.99", 4)) )
 	  {
-	    struct read_handler *new;	  
-#if WITH_SSH1_FALLBACK
-	    if (closure->fallback)
-	      {
-		assert(mode == CONNECTION_SERVER);
-	      
-		/* Sending keyexchange packet was delayed. Do it now */
-		send_kexinit(connection);
-	      }
-#endif /* WITH_SSH1_FALLBACK */
+	    struct read_handler *new; 
 
 	    connection->peer_flags = compat_peer_flags(swver_len, swver);
 
-	    new = 
-	      make_read_packet(
-		make_packet_unpad(
-		  connection,
-		  make_packet_inflate(
+	    new = make_read_packet(
 		    make_packet_debug(&connection->super,
 				      (connection->debug_comment
 				       ? ssh_format("%lz received", connection->debug_comment)
 				       : ssh_format("Received"))),
-		    connection
-		    )
-		  ),
-		connection
-		);
+		    connection);
 	    
 	    connection->versions[!mode]
 	      = ssh_format("%ls", length, line);
@@ -220,20 +197,6 @@ do_line(struct line_handler **h,
 	    *r = new;
 	    return;
 	  }
-#if WITH_SSH1_FALLBACK      
-      else if (closure->fallback
-	       && (protover_len >= 2)
-	       && !memcmp(protover, "1.", 2))
-	{
-	  *h = NULL;
-	  SSH1_FALLBACK(closure->fallback,
-			closure->fd,
-			length, line,
-			e);
-
-	  return;
-	}
-#endif /* WITH_SSH1_FALLBACK */
       else
 	{
 	  werror("Unsupported protocol version: %ps\n",
@@ -255,31 +218,39 @@ do_line(struct line_handler **h,
       *h = NULL;
       
       PROTOCOL_ERROR(connection->e,
-		     "Incorrectly version string.");
+		     "Incorrectly formatted version string.");
       
       return;
     case -1:
-      /* Display line */
-      werror("%ps\n", length, line);
+      /* If we are a client, display line */
+      if ( (connection->flags & CONNECTION_MODE) == CONNECTION_CLIENT)
+	{
+	  werror("%ps\n", length, line);
+	  
+	  /* Read next line */
+	  break;
+	}
+      else
+	{
+	  werror("Client tries to send a banner string. Disconnecting.\n");
+	  KILL(closure);
+	  *h = NULL;
 
-      /* Read next line */
-      break;
+	  PROTOCOL_ERROR(connection->e, "Invalid handshake line.");
+	  return;
+	}
     default:
       fatal("Internal error!\n");
     }
 }
 
 static struct read_handler *
-make_connection_read_line(struct ssh_connection *connection, 
-			  int fd,
-			  struct ssh1_fallback *fallback)
+make_connection_read_line(struct ssh_connection *connection)
 {
   NEW(connection_line_handler, closure);
 
   closure->super.handler = do_line;
   closure->connection = connection;
-  closure->fd = fd;
-  closure->fallback = fallback;
   return make_read_line(&closure->super, connection->e);
 }
 
@@ -288,10 +259,11 @@ struct handshake_info *
 make_handshake_info(enum connection_flag flags,
 		    const char *id_comment,
 		    const char *debug_comment,
-		    UINT32 block_size,
+		    uint32_t block_size,
 		    struct randomness *r,
 		    struct alist *algorithms,
-		    struct ssh1_fallback *fallback)
+		    struct make_kexinit *kexinit,
+		    struct lsh_string *banner_text)
 {
   NEW(handshake_info, self);
   self->flags = flags;
@@ -300,7 +272,8 @@ make_handshake_info(enum connection_flag flags,
   self->block_size = block_size;
   self->random = r;
   self->algorithms = algorithms;
-  self->fallback = fallback;
+  self->kexinit = kexinit;
+  self->banner_text = banner_text;
 
   return self;
 }
@@ -325,15 +298,18 @@ DEFINE_COMMAND4(handshake_command)
       struct command_continuation *c,
       struct exception_handler *e)
 {
-  CAST(handshake_info, info, a1);
-  CAST_SUBTYPE(make_kexinit, init, a2);
+  CAST_SUBTYPE(resource, resource, a1);
+  CAST(handshake_info, info, a2);
+  /* NOTE: For lsh, the lv from connect is mostly bogus, lv->peer is
+   * NULL */
   CAST(listen_value, lv, a4);
   
   struct lsh_string *version;
   struct ssh_connection *connection;
   int mode = info->flags & CONNECTION_MODE;
-  
-  verbose("Initiating handshake with %S\n", lv->peer->ip);
+
+  if (lv->peer)
+    verbose("Initiating handshake with %S\n", lv->peer->ip);
   
   switch (mode)
     {
@@ -344,22 +320,10 @@ DEFINE_COMMAND4(handshake_command)
 			   info->id_comment);
       break;
     case CONNECTION_SERVER:
-#if WITH_SSH1_FALLBACK
-      if (info->fallback)
-	{
-	  version =
-	    ssh_format("SSH-%lz-%lz %lz",
-		       SSH1_SERVER_PROTOCOL_VERSION,
-		       SOFTWARE_SERVER_VERSION,
-		       info->id_comment);
-	}
-      else
-#endif
-	version =
-	  ssh_format("SSH-%lz-%lz %lz",
-		     SERVER_PROTOCOL_VERSION,
-		     SOFTWARE_SERVER_VERSION,
-		     info->id_comment);
+      version = ssh_format("SSH-%lz-%lz %lz",
+			   SERVER_PROTOCOL_VERSION,
+			   SOFTWARE_SERVER_VERSION,
+			   info->id_comment);
       break;
     default:
       fatal("do_handshake: Internal error\n");
@@ -379,19 +343,27 @@ DEFINE_COMMAND4(handshake_command)
      lv->peer, lv->local, info->debug_comment, 
      make_exc_finish_read_handler(lv->fd, e, HANDLER_CONTEXT));
 
+  /* Adopt resources associated with the connection. */
+  if (resource)
+    remember_resource(connection->resources, resource);
+  
   connection_after_keyexchange(connection, c);
   
   connection_init_io
     (connection, 
-     &io_read_write(lv->fd,
-		    make_buffered_read
-		    (BUF_SIZE,
-		     make_connection_read_line(connection, 
-					       lv->fd->fd, info->fallback)),
-		    info->block_size,
-		    make_connection_close_handler(connection))
-     ->write_buffer->super,
-     info->random);
+     io_read_write(lv->fd,
+		   make_buffered_read
+		   (BUF_SIZE,
+		    make_connection_read_line(connection)),
+		   info->block_size,
+		   make_connection_close_handler(connection)));
+
+  connection->write_packet =
+    make_packet_debug(make_write_packet(connection, info->random,
+					&connection->socket->write_buffer->super),
+		      (info->debug_comment
+		       ? ssh_format("%lz sent", info->debug_comment)
+		       : ssh_format("Sent")));
 
   /* Install timeout. */
   connection_set_timeout(connection,
@@ -399,25 +371,17 @@ DEFINE_COMMAND4(handshake_command)
 			 "Handshake timed out");
   
   connection->versions[mode] = version;
-  connection->kexinit = init; 
+  connection->kexinit = info->kexinit; 
   connection->dispatch[SSH_MSG_KEXINIT]
     = make_kexinit_handler(extra, info->algorithms);
 
-#if WITH_SSH1_FALLBACK
-  /* In this mode the server SHOULD NOT send carriage return character (ascii
-   * 13) after the version identification string.
-   *
-   * Furthermore, it should not send any data after the identification string,
-   * until the client's identification string is received. */
-  if (info->fallback)
+  if (info->banner_text)
     {
-      A_WRITE(connection->raw,
-	      ssh_format("%lS\n", version));
-      return;
+      A_WRITE(&connection->socket->write_buffer->super, 
+	      ssh_format("%lS\r\n", info->banner_text));
     }
-#endif /* WITH_SSH1_FALLBACK */
 
-  A_WRITE(connection->raw,
+  A_WRITE(&connection->socket->write_buffer->super,
 	  ssh_format("%lS\r\n", version));
   
   send_kexinit(connection);
