@@ -1,6 +1,5 @@
 /* server_userauth.h
  *
- * $Id: server_userauth.h,v 1.20 2002/03/15 09:12:37 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -25,6 +24,11 @@
 #ifndef LSH_SERVER_USERAUTH_H_INCLUDED
 #define LSH_SERVER_USERAUTH_H_INCLUDED
 
+/* For uid_t and gid_t */
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include "alist.h"
 #include "command.h"
 #include "connection.h"
@@ -32,12 +36,6 @@
 #include "parse.h"
 #include "userauth.h"
 
-/* For uid_t and gid_t */
-
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #define GABA_DECLARE
 #include "server_userauth.h.x"
 #undef GABA_DECLARE
@@ -46,7 +44,7 @@
    (class
      (name user_db)
      (vars
-       ;; FIXME: Do we really need the FREE parameter?
+       ; The FREE parameter is needed only by server_publickey.c
        (lookup method "struct lsh_user *"
                       "struct lsh_string *name" "int free")))
 */
@@ -58,6 +56,11 @@ make_unix_user_db(struct reaper *reaper,
 		  const char *pw_helper, const char *login_shell,
 		  int allow_root);
 
+struct lsh_user *
+make_unix_user_self(struct lsh_string *name,
+		    struct reaper *reaper,
+		    const char *home, const char *shell);
+
 /* GABA:
    (class
      (name userauth)
@@ -66,7 +69,7 @@ make_unix_user_db(struct reaper *reaper,
                      "struct ssh_connection *connection"
 		     ; The name is consumed by this function
 		     "struct lsh_string *username"
-		     "UINT32 service"
+		     "uint32_t service"
 		     "struct simple_buffer *args"
 		     "struct command_continuation *c"
 		     "struct exception_handler *e")))
@@ -95,8 +98,6 @@ format_userauth_success(void);
 
 /* authentication methods */
 
-extern struct userauth server_userauth_none;
-
 struct userauth *
 make_userauth_password(struct user_db *db);
 
@@ -104,6 +105,11 @@ struct userauth *
 make_userauth_publickey(struct user_db *db,
 			struct alist *verifiers);
 
+struct userauth server_userauth_none_preauth;
+
+struct userauth *
+make_userauth_none_permit(struct lsh_user *user);
+
 struct command *
 make_userauth_service(struct int_list *advertised_methods,
 		      struct alist *methods,
@@ -116,4 +122,5 @@ make_exc_userauth_handler(struct ssh_connection *connection,
 			  struct exception_handler *parent,
 			  const char *context);
 
+
 #endif /* LSH_SERVER_USERAUTH_H_INCLUDED */
